;===============================================================================
;
; relics damage 1.1
;
; - allows relics that raises damage to sum up the damage bonus
;
;===============================================================================
header
hirom

;===============================================================================
;
; damage bonus
;
;===============================================================================
org     $C20D4A

;-------------------------------------------------------------------------------
main:
;-------------------------------------------------------------------------------

    ;init ----------------------------------------------------------------------
        LDA #$01            ;physic damage flag
        STA $EE             ;setup attack as physic
        STZ $EF             ;setup counter as zero

    ;check if magic damage
        LDA $11A2           ;attack flags
        LSR                 ;check physic/magic flag
        BCS .skip_magic     ;skip magic setting if physic attack

    ;setup magic setting
        ASL $EE             ;setup attack as magic
.skip_magic

    ;equipment check -----------------------------------------------------------
    ;init

        LDA $3CD0,X         ;relic 0 id
        JSR flag_counter    ;update counter
        LDA $3CD1,X         ;relic 1 id
        JSR flag_counter    ;update counter
        
    ;update damage -------------------------------------------------------------

    ;update damage
        JSL damage_update   ;update damage

    ;exit ----------------------------------------------------------------------
        RTS

;-------------------------------------------------------------------------------
flag_counter:
; A in      = item id
;-------------------------------------------------------------------------------

    ;init
        PHX

    ;check null item
        CMP #$FF            ;null item
        BEQ .exit

    ;get item offset
        XBA
        LDA #$1E            ;size of item block
        JSR $4781           ;multiply A * B
    
    ;get equipment flag
        PHP                 ;backup status
        REP #$30            ;2AB, 2XY mode
        TAX                 ;X = item offset
        LDA $D85009,X       ;get item flags
        PLP

    ;compare with attack bonus
        AND $EE             ;raise physic or magic damage flags
        BEQ .skip_counter   ;skip counter if flags don't raise attack damage

    ;increase damage
        INC $EF             ;increment damage
.skip_counter

    ;exit
.exit
        PLX 
        RTS

warnpc  $C20D86

;===============================================================================
;
; auxiliary functions
;
;===============================================================================
org     $C3FC50

;-------------------------------------------------------------------------------
damage_update:
;Y in   = damage bonus: [0, 2]
;-------------------------------------------------------------------------------

    ;init
        PHY
        PHP
        LDY $EF             ;Y = bonus counter
        BEQ .exit           ;exit if no bonus

    ;check attack flags --------------------------------------------------------
        LDA $11A2           ;flags
        AND #$20            ;ignore defense
        BNE .exit           ;exit if with flag
        LDA $11A3           ;flags
        AND #$80            ;concern mp
        BNE .exit           ;exit if with flag
        LDA $11A4           ;flags
        AND #$80            ;use hp fraction
        BNE .exit           ;exit if with flag

    ;setup 0.25 of damage ------------------------------------------------------
        REP #$20            ;mode 2AB
        LDA $11B0           ;actual damage
        LSR                 ;damage = damage / 2
        LSR                 ;damage = damage / 4
        STA $EE             ;save 0.25 of damage

    ;get fractions of damage bonus ---------------------------------------------
        TDC                 ;clean 2AB
.loop_fraction
        DEY                 ;decrement counter
        BMI .skip_fraction

    ;add fraction
        ADC $EE             ;add fraction
        BCS .damage_cap
        BRA .loop_fraction
.skip_fraction

    ;update damage -------------------------------------------------------------
        ADC $11B0           ;add bonus damage with actual damage
        BCC .skip_cap       ;don't cap damage if it doesn't overflow
.damage_cap
        LDA #$FFFF          ;maximum damage
.skip_cap
        STA $11B0           ;save damage
    
    ;exit
.exit
        PLP
        PLY
        RTL

warnpc  $C3FC8C
